'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var nodent = _interopDefault(require('nodent'));
var map = _interopDefault(require('lodash/map'));
var indent = _interopDefault(require('indent'));
var isFunction = _interopDefault(require('lodash/isFunction'));
var inspectf = _interopDefault(require('inspect-f'));
var kebabCase = _interopDefault(require('lodash/kebabCase'));
var camelCase = _interopDefault(require('lodash/camelCase'));
var upperFirst = _interopDefault(require('lodash/upperFirst'));
var flowRight = _interopDefault(require('lodash/flowRight'));
var hljs = _interopDefault(require('highlight.js'));
var Prism = _interopDefault(require('prismjs'));
var vueify = require('vueify');
var prettier = _interopDefault(require('prettier'));

var Module = require('module');
var gunzipSync = require('zlib').gunzipSync;
var join = require('path').join;
var readFileSync = require('fs').readFileSync;
var filename = join(__dirname, '../assets/prepack.min.js.gz');
var content = gunzipSync(readFileSync(filename)).toString();
var code = "\n" + content + "\nmodule.exports = Prepack\n";
var mod = new Module(filename, module.parent);
mod._compile(code, filename);
var ref = mod.exports;
var prepackSources = ref.prepackSources;
function prepack(ref) {
    var content = ref.content;
    var prefix = ref.prefix;
    var suffix = ref.suffix;
    var globalName = ref.globalName;

    var sources = [{
        filePath: '',
        sourceMapContents: '',
        fileContents: content
    }];
    var ref$1 = prepackSources(sources);
    var code = ref$1.code;
    var str = '}).call(this);';
    var start = code.lastIndexOf(str);
    var end = start + str.length;
    if (start === -1) {
        return [prefix,code,suffix].join('\n');
    }
    return [prefix,code.slice(0, start),("}).call(" + globalName + ");"),code.slice(end, -1),
        suffix].join('\n');
}

var pascalCase = flowRight(upperFirst, camelCase);
function toJSON(obj) {
    if (typeof obj === 'function') {
        var str = inspectf(2, obj);
        if (!/=>/.test(str) && /^function /.test(str) === false) {
            str = 'function ' + str;
        }
        return str;
    }
    if (obj && typeof obj === 'object') {
        if (Array.isArray(obj)) {
            return ("[" + (obj.map(toJSON).join(',\n')) + "]");
        }
        var arr = Object.keys(obj).map(function (key) { return ("\"" + key + "\": " + (toJSON(obj[key]))); });
        return ("{" + (arr.join(',')) + "}");
    }
    return JSON.stringify(obj);
}

function defer() {
    var deferred = {};
    var promise = new Promise(function (resolve, reject) {
        deferred.resolve = resolve;
        deferred.reject = reject;
    });
    deferred.promise = promise;
    return deferred;
}

var reStyle = /<style(|\s*[^>]+)>([\s\S]+)<\/style>/;
var reScript = /<script(|\s*[^>]+)>([\s\S]+)<\/script>/;
var reTemplate = /<template(|\s*[^>]+)>([\s\S]+)<\/template>/;
var extractSFC = function (code) {
    var styleMatch = reStyle.exec(code);
    var scriptMatch = reScript.exec(code);
    var templateMatch = reTemplate.exec(code);
    var style = styleMatch ? styleMatch[2] : '';
    var styleAttrs = styleMatch ? styleMatch[1].trim() : '';
    var script = scriptMatch ? scriptMatch[2].trim() : '';
    var scriptAttrs = scriptMatch ? scriptMatch[1].trim() : '';
    var template = templateMatch ? templateMatch[2].trim() : '';
    var templateAttrs = templateMatch ? templateMatch[1].trim() : '';
    if (template === '') {
        template = code.replace(reStyle, '').replace(reScript, '');
    }
    if (script === '') {
        script = 'module.exports = {}';
    } else {
        script = script.replace(/export\s+default/, 'module.exports =');
    }
    return {
        style: style,
        styleAttrs: styleAttrs,
        script: script,
        scriptAttrs: scriptAttrs,
        template: template,
        templateAttrs: templateAttrs,
        demoOnly: templateAttrs.indexOf('demo-only') > -1
    };
};

var highlightJs = function (code, lang) { return hljs.highlight(lang, code).value; };

var prism = function (code, lang) {
    if ( lang === void 0 ) lang = 'autoit';

    if (!Prism.languages[lang]) {
        try {
            require(("prismjs/components/prism-" + lang));
        } catch (e) {
            lang = 'autoit';
            require('prismjs/components/prism-autoit');
        }
    }
    return Prism.highlight(code, Prism.languages[lang]);
};
global.Prism = Prism;

var remark = require('remark');
var html = require('remark-html');
var visit = require('unist-util-visit');
var highlightMap = {
    prism: prism,
    'highlight.js': highlightJs
};
var wrapCode = function (code, lang) { return ("<pre v-pre class=\"lang-" + lang + "\"><code>" + code + "</code></pre>"); };
var highlight = function (hl) { return function (node) {
    var fn = null;
    if (isFunction(hl)) {
        fn = hl;
    } else if (highlightMap[hl]) {
        fn = highlightMap[hl];
    } else {
        throw new Error('Invalid highlight option!');
    }
    return fn(node.value, node.lang);
}; };
var transform = function (source, config) {
    var id = 0;
    var appComponents = [];
    var hl = highlight(config.highlight);
    var inject = config.inject;
    var remarkPlugins = config.remarkPlugins;
    var codeVisitor = function (node) {
        var lang = node.lang;
        var value = node.value;
        if (lang === null) {
            node.type = 'html';
            return;
        }
        var highlighted = hl(node);
        if (lang !== 'html') {
            node.type = 'html';
            node.value = wrapCode(highlighted, lang);
            return;
        }
        var ref = extractSFC(value);
        var style = ref.style;
        var styleAttrs = ref.styleAttrs;
        var script = ref.script;
        var scriptAttrs = ref.scriptAttrs;
        var template = ref.template;
        var templateAttrs = ref.templateAttrs;
        var demoOnly = ref.demoOnly;
        scriptAttrs = scriptAttrs || 'lang="buble"';
        templateAttrs = templateAttrs || 'lang="html"';
        var sfc = "\n<template " + templateAttrs + ">\n<div class=\"vue-demo\">\n" + (indent(template)) + "\n</div>\n</template>\n\n<script " + scriptAttrs + ">\n" + script + "\n</script>";
        if (style !== '') {
            styleAttrs = styleAttrs ? (styleAttrs + " ") : '';
            sfc = "<style " + styleAttrs + "scoped>" + style + "</style>\n" + sfc;
        }
        var tag = "md2vuedemo" + ((id++).toString(36));
        var className = 'vue-demo-block';
        if (demoOnly) {
            className += ' vue-demo-block-demo-only';
        }
        node.type = 'html';
        node.value = "\n<div class=\"" + className + "\">\n<" + tag + "></" + tag + ">\n" + (demoOnly ? '' : isFunction(inject) ? inject() : inject) + "\n" + (demoOnly ? '' : wrapCode(highlighted, lang)) + "\n</div>\n";
        appComponents.push({
            tag: tag,
            vue: sfc
        });
    };
    var deferred = defer();
    var transform = remark().use(html).use(remarkPlugins).use(function () { return function (ast) { return visit(ast, 'code', codeVisitor); }; });
    transform.process(source, function (err, file) {
        if (err) {
            deferred.reject(err);
        } else {
            deferred.resolve({
                appComponents: appComponents,
                html: String(file)
            });
        }
    });
    return deferred.promise;
};

var kEvent = 'style';
var StyleBundler = function StyleBundler() {
    this.CSSText = '';
    this.onStyleReceived = this.onStyleReceived.bind(this);
};
StyleBundler.prototype.onStyleReceived = function onStyleReceived (ref) {
        var style = ref.style;

    this.CSSText += '\n' + style;
};
StyleBundler.prototype.reset = function reset () {
    this.CSSText = '';
    if (this.source) {
        this.source.removeListener(kEvent, this.onStyleReceived);
    }
};
StyleBundler.prototype.setSource = function setSource (source) {
    this.reset();
    this.source = source;
    source.on(kEvent, this.onStyleReceived);
    return this;
};
StyleBundler.prototype.readOnce = function readOnce (fn) {
    var CSSText = this.CSSText.trim();
    if (typeof fn === 'function') {
        fn(CSSText);
    }
    this.reset();
    return CSSText;
};

StyleBundler.from = (function (source) { return new StyleBundler().setSource(source); });

vueify.compiler.applyConfig({
    extractCSS: true,
    customCompilers: {
        buble: function buble(content, cb) {
            var ref = require('buble').transform(content);
            var code = ref.code;
            var ret = code.replace(/\n{2,}/g, '\n').replace(/^\s+/, '  ').replace(/\s+$/, '');
            cb(null, ret);
        }
    }
});
var queue = [];
var currentTask = null;
vueify.compiler.compilePromise = (function (content, filePath) {
    if ( content === void 0 ) content = '';
    if ( filePath === void 0 ) filePath = '';

    var deferred = defer();
    var task = function () {
        var bundler = StyleBundler.from(vueify.compiler);
        currentTask = deferred.promise;
        vueify.compiler.compile(content, filePath, function (err, result) {
            if (err) {
                deferred.reject(err);
            } else {
                deferred.resolve({
                    script: result,
                    style: bundler.readOnce()
                });
            }
            process.nextTick(function () {
                currentTask = null;
                if (queue.length) {
                    queue.shift()();
                }
            });
        });
    };
    if (currentTask) {
        queue.push(task);
    } else {
        task();
    }
    return deferred.promise;
});

var md2sfc = function (arg) {
    var html = arg.html;
    var style = arg.style;
    var script = arg.script;
    var components = arg.components;
    var docInfo = arg.docInfo; if ( docInfo === void 0 ) docInfo = {};
    var stylePart = style ? ("\n<style>\n" + style + "\n</style>") : '';
    var etended = prettify(("module.exports = " + (toJSON(docInfo)) + ";"));
    return ("<template>\n<article class=\"markdown-body\">\n" + html + "\n</article >\n</template>\n\n<script lang=\"buble\">\n" + script + "\n" + etended + "\nmodule.exports.components = {\n" + (indent(components, 2)) + "\n}\n</script>\n" + stylePart + "\n");
};

function prettify(code) {
    return prettier.format(code, {
        singleQuote: true,
        parser: 'babylon'
    });
}

var compileMdSFC = function (ref) {
    var name = ref.name;
    var script = ref.script;
    var style = ref.style;

    var componentName = kebabCase(name);
    var injectCSS = style && ("\nvar insert = function (css) {\n  if (typeof window === 'undefined' || typeof document === 'undefined') return;\n  var elem = document.createElement('style')\n  elem.setAttribute('type', 'text/css')\n  elem.innerHTML = css\n\n  var head = document.getElementsByTagName('head')[0]\n  head.appendChild(elem)\n  return function () {\n    head.removeChild(elem)\n  }\n}\nexports.created = function () {\n  var css = " + (JSON.stringify(style)) + "\n  this.__clean = insert(css)\n}\nexports.destroyed = function () {\n  this.__clean()\n}\n");
    var varName = pascalCase(componentName);
    var raw = "\n" + varName + " = (function (module) {\n" + script + "\nvar exports = module.exports\nexports.name = \"" + componentName + "\"\n" + (injectCSS || '') + "\nexports.install = function (Vue) {\n  Vue.component(exports.name, exports)\n}\nreturn module.exports;\n})({});\n";
    return prepack({
        content: raw,
        prefix: 'var __proxy = {};',
        suffix: ("var component = __proxy." + varName + ";\ntypeof exports === 'object' && typeof module !== 'undefined' && (module.exports = component);\ntypeof window !== 'undefined' && window.Vue && Vue.use(component);\n"),
        globalName: '__proxy'
    });
};

nodent();
var defaults = {
    target: 'vue',
    highlight: 'highlight.js',
    inject: ''
};
var index = function (source, config) {
    if ( config === void 0 ) config = {};

    return (function ($return, $error) {
        var name, target, extend, html, appComponents, vueApps, content, script, style;
        checkDeprecation(config);
        config = Object.assign({}, defaults, config);
        var assign;
        ((assign = config, name = assign.name, target = assign.target, extend = assign.extend));
        return transform(source, config).then((function ($await_4) {
            var assign;
            ((assign = $await_4, html = assign.html, appComponents = assign.appComponents));
            vueApps = [];
            var $Loop_2_local;
            function $Loop_2_step() {
                var ref = $Loop_2_local();
                var vue = ref[0];
                var tag = ref[1];
                var $iterator_vue_tag_1 = ref[2];
                return $Loop_2.bind(this, vue, tag, $iterator_vue_tag_1);
            }
            
            function $Loop_2(vue, tag, $iterator_vue_tag_1) {
                $Loop_2_local = function () {
                    return [vue,tag,$iterator_vue_tag_1];
                };
                var assign;
                if (!($iterator_vue_tag_1[1] = $iterator_vue_tag_1[0].next()).done && ((((assign = $iterator_vue_tag_1[1].value, vue = assign.vue, tag = assign.tag, assign))) || true)) {
                    var script, style;
                    return compileVue(vue, tag).then((function ($await_5) {
                        var assign;
                        ((assign = $await_5, script = assign.script, style = assign.style));
                        vueApps.push({
                            style: style,
                            script: wrapVueCompiled({
                                tagName: tag,
                                compiled: script
                            })
                        });
                        return $Loop_2_step;
                    }).$asyncbind(this, $error), $error);
                } else 
                    { return [1]; }
            }
            
            return Function.$asyncbind.trampoline(this, $Loop_2_exit, $Loop_2_step, $error, true)($Loop_2.bind(this, undefined, undefined, [appComponents[Symbol.iterator]()]));
            function $Loop_2_exit() {
                content = md2sfc({
                    html: html,
                    docInfo: extend,
                    style: map(vueApps, 'style').join('\n'),
                    script: map(vueApps, 'script').join('\n'),
                    components: map(appComponents, function (ref) {
                        var tag = ref.tag;

                        return ("'" + tag + "': " + tag);
                }).join(',\n')
                });
                if (!target || target === 'vue') {
                    return $return(content);
                }
                if (!name) {
                    return $error(new Error('[Error] `name` must be specified!'));
                }
                return compileVue(content, name).then((function ($await_6) {
                    var assign;
                    ((assign = $await_6, script = assign.script, style = assign.style));
                    return $return(compileMdSFC({
                        script: script,
                        style: style,
                        name: name
                    }));
                }).$asyncbind(this, $error), $error);
            }
            
        }).$asyncbind(this, $error), $error);
    }).$asyncbind(this, true);
};

function checkDeprecation(config) {
    if (config.componentName) {
        console.warn('`componentName` is deprecated, use `name` instead.');
        if (typeof config.name === 'undefined') {
            config.name = config.componentName;
        }
    }
    if (config.documentInfo) {
        console.warn('`documentInfo` is deprecated, use `extend` instead.');
        if (typeof config.extend === 'undefined') {
            config.extend = config.documentInfo;
        }
    }
    if (config.customMarkups) {
        console.warn('`customMarkups` is deprecated, use `inject` instead.');
        if (typeof config.inject === 'undefined') {
            config.inject = config.customMarkups;
        }
    }
}

function compileVue(content, path) {
    return vueify.compiler.compilePromise(content, path);
}

function wrapVueCompiled(ref) {
    var tagName = ref.tagName;
    var compiled = ref.compiled;

    var code = tagName + " = (function (module) {\n" + compiled + "\nreturn module.exports\n})({});\n";
    return prepack({
        content: code,
        prefix: ("var " + tagName + " = {};"),
        suffix: (tagName + " = " + tagName + "." + tagName + ";\n"),
        globalName: tagName
    });
}

module.exports = index;
